@@ -1,5 +1,6 @@
 import datetime
 import pytz
+import json
 
 PROFILE_VALUE_POSTFIX = '_v'
 PROFILE_VALUE_UPDATED_POSTFIX = '_u'
@@ -114,6 +115,10 @@ class ProfileProperty(object):
 
     @classmethod
     def value_from_redis(cls, value):
+        return value
+
+    @classmethod
+    def value_to_redis(cls, value):
 
         return value
 
@@ -124,23 +129,33 @@ class ProfileProperty(object):
             return self.expire_at < pytz.utc.localize(datetime.datetime.utcnow())
         return False
 
+    @property
+    def output_value(self):
+        return self.value
+
     @property
     def redis_keys(self):
 
         s = self.prefix + self.name
         d = {'key': s}
         if not self.value is None:
-            d.update({'value': self.value})
+            d.update({'value': self.value_to_redis(self.value)})
         if self.expire_at:
             d.update({'expire_at': self.expire_at.timestamp()})
+        if self.extra_redis_args:
+            d.update(self.extra_redis_args)
         return d
 
+    @property
+    def extra_redis_args(self):
+        return {}
+
     @property
     def flat_redis_keys(self):
 
         s = self.prefix + self.name
         d = {
-            s + PROFILE_VALUE_POSTFIX: self.value,
+            s + PROFILE_VALUE_POSTFIX: self.value_to_redis(self.value),
             s + PROFILE_VALUE_UPDATED_POSTFIX: self.updated.timestamp(),
             s + PROFILE_VALUE_CREATED_POSTFIX: self.created.timestamp()
         }
@@ -151,14 +166,14 @@ class ProfileProperty(object):
     @property
     def short_repr(self):
 
-        return {self.name: self.value}
+        return {self.name: self.output_value}
 
     @property
     def dict_repr(self):
 
         d = {
             'name': self.name,
-            'value': self.value,
+            'value': self.output_value,
             'updated': self.updated.isoformat(),
             'created': self.created.isoformat()
         }
@@ -269,6 +284,73 @@ class LargestValue(object):
             return self.copy_off(other, expire_at=expire_at)
         return self.copy_off(self, expire_at=expire_at)
 
+
+class CounterWithTimeWindowMixin(object):
+
+    window_step_size = datetime.timedelta(days=1)
+    count_from = datetime.datetime(2021, 1, 1, 0, 0, 0)
+    timezone = pytz.timezone('Europe/Amsterdam')
+
+    @classmethod
+    def threshold_date(cls):
+
+        return pytz.utc.localize(datetime.datetime.now()).astimezone(cls.timezone) \
+               - cls.window_size * cls.window_step_size
+
+    @classmethod
+    def threshold_day_counter(cls):
+
+        return (cls.threshold_date().date() - cls.count_from.date()).days
+
+    @classmethod
+    def current_day_counter(cls):
+
+        return (pytz.utc.localize(datetime.datetime.now()).astimezone(cls.timezone) - cls.count_from.date()).days
+
+    @classmethod
+    def _current_value(cls, value):
+
+        return {ts: v for ts,v in value.items() if int(ts) > cls.threshold_day_counter()}
+
+    @property
+    def current_value(self):
+
+        return self._current_value(value=self.value)
+
+    @classmethod
+    def value_from_redis(cls, value):
+        value = value.decode() if isinstance(value, bytes) else value
+        data = {int(ts): int(val) for ts, val in json.loads(value).items()}
+        return cls._current_value(data)
+
+    @classmethod
+    def value_to_redis(cls, value):
+
+        return json.dumps(cls._current_value(value))
+
+    @property
+    def output_value(self):
+
+        return sum(self.current_value.values())
+
+    def combine(self, other):
+
+        new_updated = max([self.updated, other.updated])
+        new_created = min([self.created, other.created])
+        new_expire_at = self.combine_expire(other)
+
+        new_value = self.current_value
+        for ts, val in other.current_value.items():
+            new_value[ts] = new_value.get(ts, 0) + val
+
+        return self.__class__(self.name, new_value, updated=new_updated, created=new_created, expire_at=new_expire_at)
+
+    @property
+    def extra_redis_args(self):
+        return {
+            'thr': self.threshold_day_counter()}
+
+
 class UsableProperty(object): pass
 
 ###### Use one of these: #######
@@ -279,6 +361,12 @@ class Counter(ProfileProperty, UsableProperty):
     py_key = 'counters'
     js_key = 'counters'
 
+class CounterWithTimeWindow30(CounterWithTimeWindowMixin, ProfileProperty, UsableProperty):
+    type = dict
+    prefix = 'tmctr30_'
+    py_key = 'time_window30_counters'
+    js_key = 'timeWindow30Counters'
+    window_size = 30
 
 class SummedFloat(ProfileFloat, UsableProperty):
     prefix = 'sumflt_'
